module Responsive exposing
    ( BaseStyle, Device(..), DeviceSpec, DeviceStyles
    , baseSpacing, lineHeight, rhythm, deviceStyle, deviceStyles, mapMaybeDeviceSpec
    , Mixin, mapMixins, mediaMixins, styleAsMixin
    )

{-| The Responsive module provides a way of specifying sizing configurations for different devices,
and for applying those to create CSS with media queries.


# Models for specifying devices and their basic responsive properties.

@docs BaseStyle, Device, DeviceSpec, DeviceStyles


# Responsive helper functions.

@docs baseSpacing, lineHeight, rhythm, deviceStyle, deviceStyles, mapMaybeDeviceSpec


# Mixins

@docs Mixin, mapMixins, mediaMixins, styleAsMixin

-}

import Array exposing (Array)
import Css
import Css.Global
import Css.Media
import Maybe.Extra



-- Devices and their properties.


{-| Defines the possible classes of device; "small", "medium", "large" or "extra large".
-}
type Device
    = Sm
    | Md
    | Lg
    | Xl


{-| Defines the font size, line height, device break point width, and wrapper size, for a device.

TODO: line height ratio is a derived property. Should really hide derived props behind a type
definition, so they cannot be set up wrongly.

-}
type alias BaseStyle =
    { device : Device
    , baseFontSize : Float
    , breakWidth : Float
    , baseLineHeight : Float
    , lineHeightRatio : Float
    , wrapperWidth : Float
    }


{-| Defines a mapping from devices to something else, that must always include a definition
for each device size.
-}
type alias DeviceSpec a =
    { sm : a
    , md : a
    , lg : a
    , xl : a
    }


{-| Specifies the base styling properties accross all devices.
-}
type alias DeviceStyles =
    DeviceSpec BaseStyle


{-| Maps a device spec with optional values into a list, where the list only contains values
for the device specs that were actually defined.
-}
mapMaybeDeviceSpec : (a -> b) -> DeviceSpec (Maybe a) -> List b
mapMaybeDeviceSpec fn spec =
    [ Maybe.map fn spec.sm
    , Maybe.map fn spec.md
    , Maybe.map fn spec.lg
    , Maybe.map fn spec.xl
    ]
        |> Maybe.Extra.values



-- Vertical rhythm.


{-| Calculates the line height for a base styling.
-}
lineHeight : BaseStyle -> Float
lineHeight deviceProps =
    max
        deviceProps.baseLineHeight
        (deviceProps.lineHeightRatio * deviceProps.baseFontSize)
        |> floor
        |> toFloat


{-| Calculates a multiple of the line height for a base styling.
-}
rhythm : BaseStyle -> Float -> Css.Px
rhythm deviceProps n =
    Css.px <| n * lineHeight deviceProps



-- Device Dependant Styling


{-| Creates a single CSS property with media queries. Media queries will be
generated for each of the devices specified.
-}
deviceStyle : DeviceStyles -> (BaseStyle -> Css.Style) -> Css.Style
deviceStyle devices styleFn =
    mapMixins (mediaMixins devices (styleFn >> styleAsMixin)) []
        |> Css.batch


{-| Creates a set of CSS properties with media queries. Media queries will be
generated for each of the devices specified.
-}
deviceStyles : DeviceStyles -> (BaseStyle -> List Css.Style) -> Css.Style
deviceStyles devices styleFn =
    mapMixins (mediaMixins devices (styleFn >> stylesAsMixin)) []
        |> Css.batch



-- Mixins


{-| A mixin is a function that adds styles into a list of styles.
-}
type alias Mixin =
    List Css.Style -> List Css.Style


{-| Turns a single CSS property into a mixin.
-}
styleAsMixin : Css.Style -> Mixin
styleAsMixin style styles =
    style :: styles


{-| Turns a set of CSS properties into a mixin.
-}
stylesAsMixin : List Css.Style -> Mixin
stylesAsMixin style styles =
    style ++ styles


{-| Applies a list of mixins to a set of CSS properties, to produce a new list of CSS properties
with all the mixins applied.

TODO: Is this right? Looks like each mixin is being added into each style, than all are being
concatentated together - which ought to lead to duplicates? I think perhaps the mixins should be
chained together, not applied single then concatenated.

-}
mapMixins : List Mixin -> List Css.Style -> List Css.Style
mapMixins mixins styles =
    List.map (\mixin -> mixin styles) mixins |> List.concat



-- Media break points.


{-| Media query to match high density devices.
-}
media2x : List Css.Style -> Css.Style
media2x styles =
    Css.Media.withMediaQuery
        [ "(-webkit-min-device-pixel-ratio: 1.3), (min-resolution: 1.3dppx)" ]
        styles


{-| Creates a media query that has its min width set to the break point for a device style.
-}
mediaMinWidthMixin : BaseStyle -> Mixin
mediaMinWidthMixin { breakWidth } =
    Css.Media.withMedia [ Css.Media.all [ Css.Media.minWidth <| Css.px breakWidth ] ]
        >> List.singleton


{-| Given a set of devices, and a function to build mixins from the device properties,
creates a list of mixins, one for each device type.

The smallest (sm) device is applied without a media query, and the larger
sizes are successively applied with media queries on their break widths.

In this way, a mixin that is dependant on device properties can be applied accross
all device. Use `mapMixins` to apply the list of mixins over a list of base styles.

-}
mediaMixins : DeviceStyles -> (BaseStyle -> Mixin) -> List Mixin
mediaMixins { sm, md, lg, xl } devMixin =
    let
        minWidthDevices =
            [ xl, lg, md ]

        minWidthMixin deviceMixin deviceProps =
            deviceMixin deviceProps
                >> mediaMinWidthMixin deviceProps

        minWidthMixins deviceMixin =
            List.map (minWidthMixin deviceMixin) minWidthDevices

        allMixins deviceMixin =
            deviceMixin sm
                :: minWidthMixins deviceMixin
    in
    allMixins devMixin



-- Responsive Spacing


{-| A globaal CSS style sheet that sets up basic spaing for text, with single
direction margins.
-}
baseSpacing : DeviceStyles -> List Css.Global.Snippet
baseSpacing devices =
    [ -- No margins on headings, the line spacing of the heading is sufficient.
      Css.Global.each
        [ Css.Global.h1
        , Css.Global.h2
        , Css.Global.h3
        , Css.Global.h4
        , Css.Global.h5
        , Css.Global.h6
        ]
        [ Css.margin3 (Css.px 0) (Css.px 0) (Css.px 0) ]

    -- Single direction margins.
    , Css.Global.each
        [ Css.Global.blockquote
        , Css.Global.dl
        , Css.Global.fieldset
        , Css.Global.ol
        , Css.Global.p
        , Css.Global.pre
        , Css.Global.table
        , Css.Global.ul
        , Css.Global.hr
        ]
        [ deviceStyle devices <|
            \deviceProps -> Css.margin3 (Css.px 0) (Css.px 0) (rhythm deviceProps 1)
        ]

    -- Consistent indenting for lists.
    , Css.Global.each
        [ Css.Global.dd
        , Css.Global.ol
        , Css.Global.ul
        ]
        [ deviceStyle devices <|
            \deviceProps -> Css.margin2 (rhythm deviceProps 1) (rhythm deviceProps 1)
        ]
    ]
